
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/python/arcor2_kinali:arcor2_kinali_dist

from setuptools import setup

setup(**{
    'author': 'Robo@FIT',
    'author_email': 'imaterna@fit.vut.cz',
    'classifiers': [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Programming Language :: Python :: 3.8',
        'Natural Language :: English',
        'Topic :: Scientific/Engineering',
    ],
    'description': 'ARCOR2 integration for Kinali services.',
    'license': 'LGPL',
    'long_description': """# arcor2_kinali
# Changelog

The format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),

## [0.11.0] - 2021-02-08

### Changed
- Added the mandatory `an` parameter to all actions.

### Fixed
- Invalid url in `move_relative_joints`.

## [0.10.0] - 2020-12-14

### Changed
- Class `KinaliRobot` divided into `Aubo` and `Simatic`.
- Added example of the main script using `Resources` class.
- Added URDF model for `Aubo`.
- Updated to for Robot 0.9.0, Search 0.7.0, Barcode 0.4.0.

## [0.9.2] - 2020-10-30

### Changed
- Call to `patch_object_actions` added to the example script.
- Slight change of `inverse_kinematics` method signature (`start_joints` and `avoid_collisions` now have default values).

## [0.9.1] - 2020-10-19

### Changed
- ARCOR2 dependency updated

## [0.9.0] - 2020-10-16
### Changed
- Clients updated to use rewritten `rest` module.
- KinaliRobot updated for Robot 0.8.0 (IK/FK methods added).
- Barcode, Robot and Search now have pose while Interaction and Statistic do not.

### Fixed
- Interaction/add_dialog now returns the selected option.
- Barcode: obsolete method `active_scanners` removed.

## [0.8.0] - 2020-09-24
### Changed
- Package is now part of the arcor2 monorepo.
- Update of ObjectTypes.""",
    'long_description_content_type': 'text/markdown',
    'name': 'arcor2_kinali',
    'python_requires': '==3.8.*',
    'version': '0.11.0',
    'package_dir': {
        '': 'src',
    },
    'packages': (
        'arcor2_kinali',
        'arcor2_kinali.object_types',
        'arcor2_kinali.scripts',
    ),
    'namespace_packages': (
    ),
    'package_data': {
        'arcor2_kinali': (
            'VERSION',
            'data/aubo/aubo_i5.urdf',
            'data/aubo/aubo_i5/meshes/collision/base_link.STL',
            'data/aubo/aubo_i5/meshes/collision/foreArm_Link.STL',
            'data/aubo/aubo_i5/meshes/collision/pedestal.STL',
            'data/aubo/aubo_i5/meshes/collision/shoulder_Link.STL',
            'data/aubo/aubo_i5/meshes/collision/upperArm_Link.STL',
            'data/aubo/aubo_i5/meshes/collision/wrist1_Link.STL',
            'data/aubo/aubo_i5/meshes/collision/wrist2_Link.STL',
            'data/aubo/aubo_i5/meshes/collision/wrist3_Link.STL',
            'data/aubo/aubo_i5/meshes/visual/base_link.DAE',
            'data/aubo/aubo_i5/meshes/visual/foreArm_Link.DAE',
            'data/aubo/aubo_i5/meshes/visual/shoulder_Link.DAE',
            'data/aubo/aubo_i5/meshes/visual/upperArm_Link.DAE',
            'data/aubo/aubo_i5/meshes/visual/wrist1_Link.DAE',
            'data/aubo/aubo_i5/meshes/visual/wrist2_Link.DAE',
            'data/aubo/aubo_i5/meshes/visual/wrist3_Link.DAE',
            'py.typed',
        ),
    },
    'install_requires': (
        'Pillow==8.1.0',
        'arcor2~=0.11.0',
        'dataclasses-jsonschema-rereleased[fast-validation]==2.12.2',
    ),
    'entry_points': {
        'console_scripts': [
            'arcor2_kinali_upload_objects=arcor2_kinali.scripts.upload_objects:main',
        ],
    },
})
